/* Program to access a fortune data base using NCP extensions */

#ifdef NWWIN
#include <windows.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include <nwcaldef.h>
#include <nwconnec.h>
#include <nwserver.h>
#include <nwncpext.h>
#include <nwmisc.h>

#include "fortune.h"

#ifdef NWWIN
/**************************************************************************/
int PASCAL WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
    LPSTR lpszCmdLine, int nCmdShow)
{
   static char szAppName[] = "Fortune";
   NWCCODE ccode;

   if ( (ccode = NWCallsInit(NULL,NULL)) != 0)
   {
      MessageBox(NULL,"Error: Unable to initialize NWCALLS!","Fortune Cookie",MB_OK); /* write fortune in message box */
      return (ccode);
   }

   NCPExtension = NCP_EXTENSION_NAME;
   Server = SERVER_NAME;

   getCmdOpts(lpszCmdLine);
   ReadFortune(); /* read fortune from the NCP extension handler */
   MessageBox(NULL,outputText,"Fortune Cookie",MB_OK); /* write fortune in message box */
   return(0);
}

#elif defined (NWDOS)
/**************************************************************************/
int main(int argc, char **argv)
{
   NWCCODE ccode;

   if ( (ccode = NWCallsInit(NULL,NULL)) != 0)
   {
      printf("Error: Unable to initialize NWCALLS!\n");
      return (ccode);
   }
   NCPExtension = NCP_EXTENSION_NAME;
   Server = SERVER_NAME;

   getopts(argc,argv);

   ReadFortune();

   printf("%s", outputText);
   return (0);
}
#endif

/**************************************************************************/
void ReadFortune(void)
{
   NWCCODE        ccode;
   NWCONN_HANDLE  conn, deleteConnection;
   DWORD          extensionID;
   WORD           fortuneLen;
   char           *sendString = "Send Todays Fortune\0";
   char           fortune[MAX_FORTUNE_LEN];
   char           *bufPtr;

   bufPtr = outputText;

   if (Server == SERVER_NAME)
   {
      /* No server name so get the default server */
      if ((ccode = NWGetDefaultConnectionID(&conn)) != 0)
      {
         bufPtr += sprintf (bufPtr, "\nUnable to get a fortune from the data base\n");
         return;
      }
   }
   else
   {
      /* Sever name was entered on the command line */
      strupr(Server);
      if ((ccode=NWGetConnectionID (Server, 0, &conn, 0)) == 0)
      {
         /* Already attached to server...don't remove when done */
         deleteConnection = 0;
      }
      else
      {
         /* Not attached to server so try to attach */
         if ((ccode = NWAttachToFileServer (Server, 0, &conn)) != 0)
         {
            bufPtr += sprintf(bufPtr, "\nError 0x%X attaching to %s file server.\n", ccode, Server);
            return;
         }
         else
         {
            /* save connection for removal after fortune is read */
            deleteConnection = conn;
         }
      }
   } /* else server name is specified */

   /* Get the NCP Extension ID from the NCP Extension Name */
   if ((ccode = NWGetNCPExtensionInfoByName(conn, NCPExtension, &extensionID,
                           NULL, NULL, NULL, NULL)) != 0)
   {
      bufPtr += sprintf (bufPtr, "\nError 0x%X opening the fortune data base.", ccode );
      if (Server == SERVER_NAME)
      {
         bufPtr += sprintf(bufPtr, "\nTry specifing a server where the fortune data base resides.");
         bufPtr += sprintf(bufPtr, "\nExample Usage : fortune <-sfortune_server> <-eNCP_Extension_name>\n"); 
      }
   }
   else
   {
      /* We have the NCP extension ID now so read the fortune */
      fortuneLen = MAX_FORTUNE_LEN;

      if ((ccode = NWNCPExtensionRequest(conn, extensionID, sendString,
            strlen(sendString) + 1, fortune, &fortuneLen)) != 0)
      {
         bufPtr += sprintf (bufPtr, "Error 0x%X trying to get a fortune from the data base.\n", ccode );
      }
      else
      {
         bufPtr += sprintf (bufPtr, "\nTodays fortune is...\n%s\n", fortune );
      }
   }

   if (deleteConnection != 0)
   {
      /* clean up the connection that we allocated */
      ccode = NWDetachFromFileServer(deleteConnection);
   }
}

#ifdef NWDOS
/**************************************************************************/
void getopts(int argc, char **argv)
{
   int    i;
   char * arg;

   for(i= 1; i < argc; i++)
   {
      arg = argv[i];
      if ( *arg++ == '-')
      switch (*arg++)
      {
         case 's':
         case 'S':
            Server = arg;
            break;

         case 'n':
         case 'N':
            NCPExtension = arg;
            break;
      }
   }
}

#elif defined NWWIN

/**************************************************************************/
void getCmdOpts(LPSTR lpszCmdLine)
{                       
   int i;   
   char FAR * ptr;
                
   ptr = lpszCmdLine;                  
   while (*ptr != '\0' && *ptr != NULL)
   {                  
      if ( *ptr++ == '-')
      {
         switch (*ptr++)
         {
            /* -s<File Server Name> */
            case 's':
            case 'S':
               i = 0;
               while (*ptr != ' ' && *ptr != '\0')
                  serverName[i++] = *ptr++;
               serverName[i] = '\0';
               Server = serverName;
               break;

            /* -n<NCP Extension Name> */
            case 'n':
            case 'N':
               i = 0;
               while (*ptr != ' ' && *ptr != '\0')
                  ncpExtensionName[i++] = *ptr++;
               ncpExtensionName[i] = '\0';
               NCPExtension = ncpExtensionName;
               break;
         }
      }
   }
}
#endif
